/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Michel-Ange ANTON
 * --------------------------------------------------------------------------
 * $Id: EditDatasourcePropertiesAction.java,v 1.3 2003/06/20 17:15:41 antonma Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.webapp.jonasadmin.resource;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.objectweb.jonas.jmx.JonasManagementRepr;
import org.objectweb.jonas.jmx.JonasObjectName;
import org.objectweb.jonas.webapp.jonasadmin.JonasBaseAction;

/**
 *
 */

public class EditDatasourcePropertiesAction extends JonasBaseAction {

// --------------------------------------------------------- Public Methods

    /**
     */
    public ActionForward executeAction(ActionMapping p_Mapping, ActionForm p_Form
        , HttpServletRequest p_Request, HttpServletResponse p_Response)
        throws IOException, ServletException {

        // Datasource to create
        String sAction = p_Request.getParameter("action");
        // Datasource to edit
        String sName = p_Request.getParameter("name");
        // Form used
        DatasourcePropertiesForm oForm = (DatasourcePropertiesForm) m_Session.getAttribute(
            "datasourcePropertiesForm");
        // Create form
        if (oForm == null) {
            oForm = new DatasourcePropertiesForm();
            m_Session.setAttribute("datasourcePropertiesForm", oForm);
        }
        // Resolve flag
        boolean bCreate = (sAction != null) && (sAction.equals("create") == true);
        boolean bPopulate = (bCreate == false) && (sName != null);
        // Reset form
        if (bCreate == true) {
            oForm.reset(p_Mapping, p_Request);
        }
        // Populate Help viewer
        oForm.setCreate(bCreate);
        // Populate
        if (bPopulate == true) {
            try {
                oForm.reset(p_Mapping, p_Request);
                populate(sName, oForm);
            }
            catch (Throwable t) {
                addGlobalError(t);
                saveErrors(p_Request, m_Errors);
                return (p_Mapping.findForward("Global Error"));
            }
        }
        // Forward to the jsp.
        return (p_Mapping.findForward("Datasource Properties"));
    }

// --------------------------------------------------------- Protected Methods

    protected void populate(String p_Name, DatasourcePropertiesForm p_Form)
        throws Exception {

        String[] asParam = new String[1];
        String[] asSignature = new String[1];
        asParam[0] = p_Name;
        asSignature[0] = "java.lang.String";

        Properties oProps = (Properties) JonasManagementRepr.invoke(JonasObjectName.databaseService()
            , "getDataSourcePropertiesFile", asParam, asSignature);
        /*
         #set ( $val = $datasourceProperties.getProperty("datasource.name", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.description", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.url", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.classname", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.username", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.password", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.factory", "none") )
         #set ( $val = $datasourceProperties.getProperty("datasource.mapper", "") )
         #set ( $val = $datasourceProperties.getProperty("datasource.connmaxage", "30") )
         #set ( $val = $datasourceProperties.getProperty("datasource.connchecklevel", "0") )
         #set ( $val = $datasourceProperties.getProperty("datasource.connteststmt", "SELECT 1") )
         #set ( $val = $datasourceProperties.getProperty("datasource.minconpool", "0") )
             #set ( $val = $datasourceProperties.getProperty("datasource.maxconpool", "no max boundary") )
         */
        // Populate
        p_Form.setName(p_Name);
        p_Form.setDatasourceName(getStringAttribute(oProps, "datasource.name"
            , p_Form.getDatasourceName()));
        p_Form.setDatasourceDescription(getStringAttribute(oProps, "datasource.description"
            , p_Form.getDatasourceDescription()));
        p_Form.setDatasourceUrl(getStringAttribute(oProps, "datasource.url"
            , p_Form.getDatasourceUrl()));
        p_Form.setDatasourceClassname(getStringAttribute(oProps, "datasource.classname"
            , p_Form.getDatasourceClassname()));
        p_Form.setDatasourceUsername(getStringAttribute(oProps, "datasource.username"
            , p_Form.getDatasourceUsername()));
        p_Form.setDatasourcePassword(getStringAttribute(oProps, "datasource.password"
            , p_Form.getDatasourcePassword()));
        p_Form.setDatasourceFactory(getStringAttribute(oProps, "datasource.factory"
            , p_Form.getDatasourceFactory()));
//      p_Form.setDatasourceXadataname(getStringAttribute(oProps, "datasource.xadataname"
//         , p_Form.getDatasourceXadataname()));
//      p_Form.setDatasourceMincon(getStringAttribute(oProps, "datasource.mincon"
//         , p_Form.getDatasourceMincon()));
//      p_Form.setDatasourceMaxcon(getStringAttribute(oProps, "datasource.maxcon"
//         , p_Form.getDatasourceMaxcon()));
//      p_Form.setDatasourceIsolationlevel(getStringAttribute(oProps, "datasource.isolationlevel"
//         , p_Form.getDatasourceIsolationlevel()));
        p_Form.setDatasourceMapper(getStringAttribute(oProps, "datasource.mapper"
            , p_Form.getDatasourceMapper()));
        p_Form.setJdbcConnmaxage(getStringAttribute(oProps, "jdbc.connmaxage"
            , p_Form.getJdbcConnmaxage()));
        p_Form.setJdbcConnchecklevel(getStringAttribute(oProps, "jdbc.connchecklevel"
            , p_Form.getJdbcConnchecklevel()));
        p_Form.setJdbcConnteststmt(getStringAttribute(oProps, "jdbc.connteststmt"
            , p_Form.getJdbcConnteststmt()));
//      p_Form.setJdbcConnexcltimeout(getStringAttribute(oProps, "jdbc.connexcltimeout"
//         , p_Form.getJdbcConnexcltimeout()));
//      p_Form.setJdbcConnmaxthreads(getStringAttribute(oProps, "jdbc.connmaxthreads"
//         , p_Form.getJdbcConnmaxthreads()));
        p_Form.setJdbcMinconpool(getStringAttribute(oProps, "jdbc.minconpool"
            , p_Form.getJdbcMinconpool()));
        p_Form.setJdbcMaxconpool(getStringAttribute(oProps, "jdbc.maxconpool"
            , p_Form.getJdbcMaxconpool()));
//      p_Form.setJdbcSleeptimepool(getStringAttribute(oProps, "jdbc.sleeptimepool"
//         , p_Form.getJdbcSleeptimepool()));
//      p_Form.setJdbcGcpool(getBooleanAttribute(oProps, "jdbcGcpool", p_Form.getJdbcGcpool()));
//      p_Form.setJdbcGenerationpool(getStringAttribute(oProps, "jdbc.gGenerationpool"
//         , p_Form.getJdbcGenerationpool()));
//      p_Form.setJdbcDeadlockpool(getStringAttribute(oProps, "jdbc.deadlockpool"
//         , p_Form.getJdbcDeadlockpool()));
//      p_Form.setJdbcLoopwaitpool(getStringAttribute(oProps, "jdbc.loopwaitpool"
//         , p_Form.getJdbcLoopwaitpool()));
    }
}
