/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: WsGen.java,v 1.9 2003/09/19 15:07:47 delplanx Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ws.wsgen;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas_ejb.lib.Version;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.List;
import java.util.Iterator;

import java.net.URL;
import java.net.MalformedURLException;
import java.net.URLClassLoader;

//import ear
import org.objectweb.jonas_ear.deployment.xml.Web;
import org.objectweb.jonas_ear.deployment.xml.Ejb;
import org.objectweb.jonas_ear.deployment.api.EarDeploymentDesc;
import org.objectweb.jonas_ear.deployment.api.EarDeploymentDescException;
import org.objectweb.jonas.ear.lib.EarFileManager;
import org.objectweb.jonas.ear.lib.EarFileManagerException;

//import ejb
import org.objectweb.jonas_ejb.deployment.lib.EjbDeploymentDescManager;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDesc;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDescException;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;

//import ws
import org.objectweb.jonas_ws.deployment.lib.WSDeploymentDescManager;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.WSDeploymentDescException;

//import web
import org.objectweb.jonas_web.deployment.lib.WebDeploymentDescManager;
import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDesc;
import org.objectweb.jonas_web.deployment.api.WebContainerDeploymentDescException;


/**
 * 
 *
 * @author Xavier Delplanque & Guillaume Sauthier
 */
public class WsGen {
    
    /** The Generator used for webservices classes and configuration files creation */
    private ConfigGenerator cg;

    private boolean keepGenerated;

    private String tmpDir;

    /** Is the command on verbose ? */
    private boolean verbose = false;

    /** Is the command is on debug mode ? */
    private boolean debug = false;

    /** Is the command used for help message ? */
    private boolean isHelp = false;

    /** Name of the directory where to place the generated file */
    private String outputdir = null;

    /** The logger to use */
    private static Logger logger = null;

    private static WsGen wsg;
    /**
     * Constructor : creates a WsGen object.
     * Generate source files for client and configuration files
     * for both endpoints and clients (if needed).
     *
     * @param outputDir the directory where to store generated files
     * @param cg The ConfigGenerator to use (Axis/Glue/...)
     * @param isVerbose Indicate if command is in verbose mode
     * @param isDebug Indicate if command is in debug mode
     */
    public WsGen(String outputDir,
		 ConfigGenerator cGen,
		 boolean isVerbose,
		 boolean isDebug){

	// throws WsGenException 		
        verbose = isVerbose;
	cg = cGen;
	debug = isDebug;
        outputdir = outputDir;
	
	if (logger == null)
	    logger = Log.getLogger(Log.JONAS_WSGEN_PREFIX);
    }

    /**
     * JOnAS Method
     */
    public static void main(String args[]) {
        boolean  error = false;

        boolean  isHelp          = false;
        boolean  isVerbose       = false;
        boolean  isDebug         = false;
        boolean  isKeepGenerated = false;
        boolean  noConfig        = false;
        boolean  parseWithValidation = true;
	String   cGenClassName = null;

        String   fileInputName = null;
        String   dirOutputName = null;
        String   nameJavac     = null;
        Vector   optionsJavaC  = new Vector();

        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String arg = args[argn];
            if (arg.equals("-help") || arg.equals("-?")) {
                isHelp = true;
                continue;
            }
            if (arg.equals("-verbose")) {
                isVerbose = true;
                continue;
            }
            if (arg.equals("-debug")) {
                isDebug = true;
                isVerbose = true;
                continue;
            }
            if (arg.equals("-keepgenerated")) {
                isKeepGenerated = true;
                continue;
            }
            if (arg.equals("-noconfig")) {
                noConfig = true;
                continue;
            }
            if (arg.equals("-novalidation")) {
                parseWithValidation = false;
                continue;
            }
            if (arg.equals("-cg")) {
                cGenClassName = args[++argn];
                continue;
            }
            if (arg.equals("-javac")) {
                argn++;
                continue;
            }
            if (arg.equals("-javacopts")) {
                argn++;
                if (argn < args.length) {
                    StringTokenizer st = new StringTokenizer(args[argn]);
                    while (st.hasMoreTokens()) {
                        optionsJavaC.addElement(st.nextToken());
                    }
                } else {
                    error = true;
                }
                continue;
            }
            if (arg.equals("-d")) {
                argn++;
                if (argn < args.length) {
                    dirOutputName = args[argn];
                } else {
                    error = true;
                }
                continue;
            }
            fileInputName = args[argn];
        }

        Log.configure("trace");
	
        logger = Log.getLogger(Log.JONAS_WSGEN_PREFIX);

        // Usage ?
        if (isHelp) {
            usage();
            return;
        }

        // Check args
        if (error || (fileInputName == null) ||
	    (cGenClassName == null) || (dirOutputName == null)) {
            usage();
            throw new RuntimeException();
        }

	File inputFile = null;
	try {	    
	    inputFile = new File(fileInputName);
	    fileInputName = inputFile.getCanonicalPath();
	} 
	catch (java.io.IOException ioe) {
	    String err = "FileInputName : " + fileInputName +
		" not found ! ";
	    WsGen.fatalError(err);	    
	}

	File outputDir = null;
	try {	    
	     outputDir = new File(dirOutputName);
	    dirOutputName = outputDir.getCanonicalPath();
	} 
	catch (java.io.IOException ioe) {
	    String err = "dirOutputName : " + dirOutputName +
		" not found ! ";
	    WsGen.fatalError(err);
	}

	// we build a copy of the given file in the given repertory
	File outFile;
	try {
	    if (!outputDir.exists()) {
		// if the given directory doesn't exist, create it.
		outputDir.mkdirs();
	    }
	    outFile = new File(dirOutputName + File.separator + inputFile.getName());
	    
	    if (!outFile.equals(inputFile)) {
		FileOutputStream os = new FileOutputStream(outFile);
		FileInputStream is = new FileInputStream(inputFile);
		// copy the file
		byte[] buf = new byte[1024];
		int length;
		while ((length = is.read(buf)) > 0)
		    os.write(buf, 0, length);	
		is.close();
		os.flush();
		os.close();
		fileInputName = outFile.getCanonicalPath();
	    }
	}
	catch (Exception e) {
	    String err = "Error when copying '" + fileInputName + 
		"' in '" + dirOutputName +
		"'";
	    WsGen.fatalError(err);
	}	
	
	// Get the ConfigGenerator class to use
	ConfigGenerator confGen = null;
	try {
	    confGen = (ConfigGenerator) Class.forName(cGenClassName).newInstance();
	    confGen.setOption(confGen.VERBOSE, new Boolean(isVerbose));
	    confGen.setOption(confGen.DEBUG, new Boolean(isDebug));
	    confGen.setOption(confGen.NO_CONFIG, new Boolean(noConfig));
	    confGen.setOption(confGen.KEEP_GEN, new Boolean(isKeepGenerated));
	}
	catch(ClassNotFoundException cnf) {
	    String err = "ConfigGenerator subClass : " + cGenClassName +
		" not found ! " + cnf;
	    WsGen.fatalError(err);
	}
	catch(InstantiationException ie) {
	    String err = "Error when instantiate : " + cGenClassName + " : " + ie;
	    WsGen.fatalError(err);
	}
	catch(IllegalAccessException iae) {
	    String err = "Error when instantiate : " + cGenClassName + " : " + iae;
	    WsGen.fatalError(err);
	}
	    
	wsg = new WsGen(dirOutputName,
			confGen,
			isVerbose,
			isDebug);

	// EAR case : deflate the ear in tempo. directory
	if (fileInputName.endsWith(".ear")) {
	    wsg.configEar(fileInputName);
	} // End EAR Case
	else if (fileInputName.endsWith(".jar")) {
	    wsg.configEjbJar(fileInputName);
	} // End EJB Case
	else if (fileInputName.endsWith(".war")) {
	    wsg.configWebApp(fileInputName);
	} // End Webapp Case
	else {
	    // if inputfile not an EAR or EJBJAR or WAR
	    usage();
            throw new RuntimeException();
	}
    }

    /**
     * Display the usage
     */
    public static void usage() {
        StringBuffer msg = new StringBuffer();
        msg.append("Usage: java org.objectweb.jonas_ws.wsgen.WsGen -help \n");
        msg.append("         to print this help message \n");
        msg.append(" or    java org.objectweb.jonas_ws.wsgen.WsGen <Options> <Input_File> \n");
        msg.append("Options include: \n");
        msg.append("       -d <output_dir>  specify where to place the generated files \n");
        msg.append("       -cg <class_name> specify the ConfigGenerator class to use \n");
        msg.append("       -novalidation    parse the XML deployment descriptors without \n");
        msg.append("                        validation \n");
        msg.append("       -javac     <opt> specify the java compiler to use \n");
        msg.append("       -javacopts <opt> specify the options to pass to the java compiler \n");
        msg.append("       -keepgenerated   do not delete intermediate generated files \n");
        msg.append("       -noconfig        do not generate configuration files (require \n");
	msg.append("                        user privided files) \n");
        msg.append("       -verbose \n");
        msg.append("       -debug \n");
        msg.append(" \n");
        msg.append("       Input_File       the ejb-jar, war or ear filename\n");
        WsGen.info(msg.toString());
    }

    /**
     * Display the specified message only if verbose.
     * @param msg the message to display
     */
    void trace(String msg) {
        logger.log(verbose ? BasicLevel.INFO : BasicLevel.DEBUG, msg);
    }

    /**
     * Display the specified message.
     * @param msg the message to display
     */
    public static void info(String msg) {
	if (logger.isLoggable(BasicLevel.INFO))
	    logger.log(BasicLevel.INFO, msg);
    }

    /**
     * Display the specified message.
     * @param msg the message to display
     */
    public static void debug(String msg) {
	if (logger.isLoggable(BasicLevel.DEBUG))
	    logger.log(BasicLevel.DEBUG, msg);
    }

    /**
     * Display the specified warning message.
     * @param msg the warning message to display
     */
    public static void warning(String msg) {
	if (logger.isLoggable(BasicLevel.WARN))
	    logger.log(BasicLevel.WARN, msg);
    }

    /**
     * Display the specified error message.
     * @param msg the error message to display
     */
    public static void error(String msg) {
	if (logger.isLoggable(BasicLevel.ERROR))
	    logger.log(BasicLevel.ERROR, msg);
    }


    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param msg the error message to display
     */
    public static void fatalError(String msg) {
        System.err.println("WsGen fatal error: " + msg);
        throw new RuntimeException(msg);
    }

    /**
     * Display the specified error message and exits with an
     * EXIT_FAILURE status.
     * @param e the error to display
     */
    public static void fatalError(Exception e) {
        System.err.println("WsGen fatal error: ");
        printException(e, System.err);
        e.printStackTrace(System.err);
        throw new RuntimeException(e.getMessage());
    }
    /**
     * Display the exception and its nested exception if exists, on the given printstream.
     * @param e the exception to display
     * @param out the PrintStream on which the exception has to displayed
     */ 
    static void printException(Exception e, PrintStream out) {
        Exception inner = null;
        if (e instanceof WsGenException
            && (inner = ((WsGenException) e).inner) != null) {
            out.println(e.getMessage());
            printException(inner, out);
        } else {
            e.printStackTrace();
        }
    }

    /**
     * Create a cleaned temporary directory.
     * @return the temp directory file
     * @throws IOException if a temp directory cannot be created.
     */
    public static File createTempDir() throws IOException {       
        File tmpDir = File.createTempFile("wsgen", null, null);    
        tmpDir.delete();
        if (!tmpDir.mkdir()) {
            throw new IOException("Cannot create the temporary directory '" + tmpDir + "'.");
        }
        return tmpDir;
    }

    /**
     * Modify the given Web App to add it Webservices files
     *
     * @param fileInputName the name of the web app file
     */
    private void configWebApp(String fileInputName){
	debug("Webapp Case : " + fileInputName);


	URL warURL = getExplodedJar(fileInputName);
	    
	Vector urls = new Vector();

	File lib = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "lib");
	File classes = new File(warURL.getFile() + File.separator + "WEB-INF" + File.separator + "classes");
	try {
	    if (lib.exists()) {
		File[] childs = lib.listFiles();
		for (int i = 0 ; i < childs.length; i++) {
		    urls.add( childs[i].toURL());
		}
	    }
	    if (classes.exists()) {
		urls.add( classes.toURL());
	    }
	}
	catch (java.net.MalformedURLException mue) {
	    wsg.fatalError("Error when building war '" + fileInputName + "' class loader");
	}
	urls.add(warURL);
	
	URL[] aurls = new URL[urls.size()];
	aurls = (URL[])urls.toArray(aurls);
	debug("-------------------");
	for (int i = 0; i < urls.size(); i++)
	    debug(urls.get(i).toString());
	debug("-------------------");
	//Create classLoader
	//parent classloader is the current classloader
	ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
	URLClassLoader fileCL = new URLClassLoader(aurls,
						   currentLoader);
	try {
	    // Endpoint files generation.
	    // get the webservices deployment descriptor.
	    WSDeploymentDesc wsdd = null;
	    try {
		wsdd = WSDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
	    }
	    catch (WSDeploymentDescException wsdde) {
		String err= "Error while reading Deployment Desc from " + fileInputName;
		throw new WsGenException(err, wsdde);
	    }
	    
	    
	    if (wsdd != null) {
		// the web app may have endpoints.
		// add webservices files in war
		cg.genEndpointConfig(wsdd, fileInputName);
		WarManager wm = new WarManager(fileInputName);
		wm.addServlet(cg.getEngineWebApp(), new Vector());
		wm.close();
	    }
	    
	    
	    // Client Files generation.
	    // get the web app deployment descriptor. 
	    WebContainerDeploymentDesc wcdd = null;
	    try {
		wcdd = WebDeploymentDescManager.getDeploymentDesc(fileInputName,fileCL);
	    }
	    catch (WebContainerDeploymentDescException wcdde) {
		throw new WsGenException(wcdde);
	    }

	    if (wcdd.getServiceRefTags().length != 0) {
		// the web app may have clients
		// add clients files in war
		cg.genClientConfig(wcdd.getServiceRefTags(), fileInputName);
	    }
	}
	catch (WsGenException wsge) {
	    wsge.printStackTrace(System.out);
	    String err = "Error in generation for " + fileInputName;
	    wsg.fatalError(err);
	}

	// remove the exploded ear
	File tempWar = new File(warURL.getFile());
	try {
	    if (!tempWar.delete()) {
		String err = "Cannot delete the war directory '" + tempWar.getName()+"'";
		wsg.warning(err);
	    }
	}
	catch (NullPointerException npe){
	    warning("Null File name given '"+tempWar.getName()+"'");
	}
	catch (SecurityException se){
	    warning("Security error when deleting '"+tempWar.getName()+"'");
	}
}
        
    
    /**
     * Modify the given EjbJar to add it Webservices files
     *
     * @param fileInputName the name of the Ejbjar file
     */
    private void configEjbJar(String fileInputName){

	wsg.debug("EjbJar Case : " + fileInputName);
	URL ejbURL = null;
	try {
	    ejbURL = new File(fileInputName).toURL();
	}
	catch (MalformedURLException mue) {
	    wsg.error(mue.getMessage());
	    wsg.fatalError(mue);
	}
	    
	//Create classLoader
	//parent classloader is the current classloader
	ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
	URLClassLoader fileCL = new URLClassLoader(new URL[] {ejbURL},
						   currentLoader);

	try {

	    // Client Files generation.
	    // get the ejb jar deployment descriptor.
	    DeploymentDesc dd = null;
	    try {
		dd = EjbDeploymentDescManager.getDeploymentDesc(fileInputName,fileCL);
	    }
	    catch (DeploymentDescException dde) {
		throw new WsGenException(dde);
	    }

	    cg.genClientConfig(dd.getBeanDescIterator(), fileInputName);
	    // Endpoint files generation.
	    // get the webservices deployment descriptor.
	    WSDeploymentDesc wsdd = null;

	    try {
		wsdd = WSDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
	    }
	    catch (WSDeploymentDescException wsdde) {
		wsdde.printStackTrace(System.out);
		String err= "Error while reading Deployment Desc from " + fileInputName;
		throw new WsGenException(err, wsdde);
	    }
	    if (wsdd != null) {
		// the ejb may have endpoints.

		// add webservices files in ejbjar
		cg.genEndpointConfig(wsdd, fileInputName);

		// create a war file and add it the WS servlet.
		String warFileName = fileInputName.substring(0, fileInputName.length()-3)+ "war";
		WarManager wm = new WarManager(warFileName);

		// todo !!! : set the mapping
		Vector map = new Vector();
		map.add(cg.getDefaultMapping());
		wm.addServlet(cg.getEngineWebApp(), map);

		wm.close();
		// create an ear and add it the jar and war.
		// the ear name is ejbjar one with '.ear' extension.
		String earName = fileInputName.substring(0, fileInputName.length()-3)+ "ear";

		EarManager em = new EarManager(earName);
		em.addWar(warFileName);
		em.addEjbjar(fileInputName);
		em.close();
		// remove the war and ejbjar files
		try {
		    File war = new File(warFileName);
		    File ejbjar = new File(fileInputName);
		    if (!war.delete()){
			warning("Cannot delete the war '"+warFileName+"'");
		    }
		    if (!ejbjar.delete()){
			warning("Cannot delete the ejbjar '"+fileInputName+"'");
		    }		    
		}
		catch (NullPointerException npe){
		    warning("Null File name given '"+warFileName+"', '"+fileInputName+"'");
		}
		catch (SecurityException se){
		    warning("Security error when deleting '"+warFileName+"', '"+fileInputName+"'");
		}
	    }		
	}
	catch (WsGenException wsge) {
	    String err = "Error in generation for " + fileInputName;
	    fatalError(err);
	}
    }

    /**
     * Modify the given EjbJar (contained in an Ear) to add it Webservices files
     *
     * @param fileInputName the name of the Ejbjar file
     */
    private void configEjbJar(EarManager em, String fileInputName){
	debug("EjbJar Case : " + fileInputName);

	URL ejbURL = null;
	try {
	    ejbURL = new File(fileInputName).toURL();
	}
	catch (MalformedURLException mue) {
	    error(mue.getMessage());
	    fatalError(mue);	
	}
	    
	//Create classLoader
	//parent classloader is the current classloader
	ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
	URLClassLoader fileCL = new URLClassLoader(new URL[] {ejbURL},
						   currentLoader);

	try {

	    // Client Files generation.
	    // get the web app deployment descriptor.
	    DeploymentDesc dd = null;
	    try {
		dd = EjbDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
	    }
	    catch (DeploymentDescException dde) {
		throw new WsGenException(dde);
	    }

	    cg.genClientConfig(dd.getBeanDescIterator(), fileInputName);

	    // Endpoint files generation.
	    // get the webservices deployment descriptor.
	    WSDeploymentDesc wsdd = null;
	    try {
		wsdd = WSDeploymentDescManager.getDeploymentDesc(fileInputName, fileCL);
	    }
	    catch (WSDeploymentDescException wsdde) {
		String err= "Error while reading Deployment Desc from " + fileInputName;
		throw new WsGenException(wsdde);
	    }

	    if (wsdd != null) {
		// the ejb may have endpoints.

		// add webservices files in ejbjar
		cg.genEndpointConfig(wsdd, fileInputName);

		// create a war file and add it the WS servlet.
		String warFileName = fileInputName.substring(0, fileInputName.length()-3)+ "war";
		WarManager wm = new WarManager(warFileName);
		// todo !!!! set the mapping 
		Vector map = new Vector();
		map.add(cg.getDefaultMapping());
		wm.addServlet(cg.getEngineWebApp(), map);
		wm.close();
		    
		// the war in the ear.
		em.addWar(warFileName);
		
		// remove the war file.
		try {
		    File war = new File(warFileName);
		    if (!war.delete()){
			warning("Cannot delete the war '"+warFileName+"'");
		    }
		}
		catch (NullPointerException npe){
		    warning("Null File name given '"+warFileName+"'");
		}
		catch (SecurityException se){
		    warning("Security error when deleting '"+warFileName+"'");
		}

	    }				
	}
	catch (WsGenException wsge) {
	    String err = "Error in generation for " + fileInputName;
	    wsg.fatalError(err);
	}
    }

    /**
     * Modify the given Ear to add it Webservices files
     *
     * @param fileInputName the name of the Ear file
     */
    private void configEar(String fileInputName){

	URL earURL = getExplodedJar(fileInputName);
	//Create classLoader
	//parent classloader is the current classloader
	ClassLoader currentLoader = Thread.currentThread().getContextClassLoader();
	URLClassLoader loaderCls = new URLClassLoader(new URL[] {earURL},
						      currentLoader);
	    
	EarDeploymentDesc earDD = null;
	debug("Getting the deployment descriptor of the file" + fileInputName);
	try {
	    earDD = EarDeploymentDesc.getInstance(fileInputName, loaderCls);
	} catch (EarDeploymentDescException e) {
	    String err = "Error in the Deployment descriptor '"
		+ fileInputName + "': " + e;
	    error(err);
	    fatalError(err);
	}

	//We get the tags from the Deployment descriptor
	Web[] webTags = earDD.getWebTags();
	Ejb[] ejbTags = earDD.getEjbTags();

	//Check if all modules are inside the EAR file
	//no relatives mode like ../../file1.jar	    
	File fEar = new File(fileInputName);
	File tmpFile = null;

	// list of files
	File[] ejbFiles = new File[ejbTags.length];
	String[] ejbEntryNames = new String[ejbTags.length];
	String[] ejbPaths = new String[ejbTags.length];
	
	File[] warFiles = new File[webTags.length];
	String[] warEntryNames = new String[webTags.length];
	String[] warPaths = new String[webTags.length];
	
	try {
	    for (int i = 0; i < ejbTags.length; i++) {
		ejbEntryNames[i] = ejbTags[i].getValue();
		ejbFiles[i] = new File(fEar, ejbEntryNames[i]);
		tmpFile = ejbFiles[i].getCanonicalFile();
		ejbPaths[i] = tmpFile.getCanonicalPath();

		if (!tmpFile.getPath().startsWith(fEar.getPath())) {
		    String err =  "Error : The ejb-jar file " +
			ejbTags[i].getValue()  + " is not inside the ear file " +
			fEar;
		    error(err);
		    fatalError(err);
		}
	    }
		
	    for (int i = 0; i < webTags.length; i++) {
		warEntryNames[i] = webTags[i].getWebUri().getValue();
		warFiles[i] = new File(fEar, warEntryNames[i]);
		tmpFile = warFiles[i].getCanonicalFile();
		warPaths[i] = tmpFile.getCanonicalPath();
		    
		if (!tmpFile.getPath().startsWith(fEar.getPath())) {
		    String err =  "Error : The war file " +
			webTags[i].getWebUri().getValue() +
			" is not inside the ear file " + fEar;
		    error(err);
		    fatalError(err);
		}
	    }
		
	} catch (IOException ioe) {
	    String err =  "Error while trying to get the canonical file of " + tmpFile;
	    error(err + " : " + ioe.getMessage());
	    fatalError(err);
	}
	    
	// We have the list of files to process now
	// give them to WsGen instance
	String filename = null;
	try {
	    EarManager earm = new EarManager(fileInputName);
	    // add Webservices files in Ejbs
	    for (int i = 0; i < ejbPaths.length; i++) {
		
		filename = ejbPaths[i];
		configEjbJar(earm,filename);
		// todo : verifier que l ancien jar est bien ecrase
		// overwrite the archive contained in the ear.
		
		earm.addFile(ejbEntryNames[i], ejbFiles[i]);
	    }
	    // add Webservices files in WebApps
	    for (int i = 0 ; i < warFiles.length; i++) {

		filename = warPaths[i];
		configWebApp(filename);
		// todo : verifier que l ancien jar est bien ecrase
		// overwrite the archive contained in the ear.
		earm.addFile(warEntryNames[i], warFiles[i]);
	    }
	    earm.close();
	}
	catch (Exception e) {
	    String err = "Error in generation for " + filename;
	    fatalError(err);
	}

	// remove the exploded ear
	File tempEAR = new File(earURL.getFile());
	try {
	    if (!tempEAR.delete()) {
		String err = "Cannot delete the ear directory '" + tempEAR.getName()+"'";
		fatalError(err);
	    }
	}
	catch (NullPointerException npe){
	    warning("Null File name given '"+tempEAR.getName()+"'");
	}
	catch (SecurityException se){
	    warning("Security error when deleting '"+tempEAR.getName()+"'");
	}
    }

    public URL getExplodedJar (String fileInputName) {
	File tempDir = null;       	
	// unpack the file inside a temporary directory
	try {
	    tempDir = createTempDir();
	}
	catch (IOException ioe) {
	    String err = "Cannot create temporary Directory !";
	    fatalError(err);
	}	    
	URL fileURL = null;
	URL temp = null;
	try {
	    fileURL = new File(fileInputName).toURL();
	    URL tempURL = tempDir.toURL();
	    temp = EarFileManager.unpackEar(fileURL, tempURL);
	}
	catch (MalformedURLException mue) {
	    error(mue.getMessage());
	    fatalError(mue);
	}
	catch (EarFileManagerException fme) {
	    error(fme.getMessage());
	    fatalError(fme);
	}
	    
	return temp;
    }
}
