/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: AxisConfigGenerator.java,v 1.7 2003/09/19 15:07:48 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen.axis;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import org.objectweb.jonas.common.Log;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;

import org.objectweb.jonas_ws.deployment.api.WSDeploymentDesc;
import org.objectweb.jonas_ws.deployment.api.ServiceRef;
import org.objectweb.jonas_ws.deployment.api.HandlerRef;
import org.objectweb.jonas_ws.deployment.api.MappingFile;

import org.objectweb.jonas_ws.wsgen.AbsConfigGenerator;
import org.objectweb.jonas_ws.wsgen.WsGenException;
import org.objectweb.jonas_ws.wsgen.WsGen;

import org.objectweb.common.Cmd;

import java.util.Iterator;
import java.util.List;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * Implement methods generating AXIS specific configuration files and classes.
 * A class is created for each WS Engine
 * for example, a new class called
 * GlueConfigGenerator for Glue integration
 *
 *
 * @author Guillaume Sauthier & Xavier Delplanque
 */
public class AxisConfigGenerator extends AbsConfigGenerator {

    private VelocityEngine vEngine;
    private int fileCptr;
    
    public AxisConfigGenerator() 
	throws WsGenException {
	super();

        String jonasRoot = System.getProperty("install.root");
        if (jonasRoot == null) {
            throw new WsGenException("System property 'install.root' not set");
        }	
        String path2Tmpl = new String(jonasRoot + File.separatorChar + "templates"
                                      + File.separatorChar + "wsgen" 
				      + File.separatorChar + "axis");
        vEngine = new VelocityEngine();
        vEngine.setProperty(VelocityEngine.VM_LIBRARY, "");
        vEngine.setProperty(VelocityEngine.RESOURCE_LOADER, "file");
        vEngine.setProperty(VelocityEngine.FILE_RESOURCE_LOADER_PATH, path2Tmpl);
        try {
            vEngine.init();
        } catch (Exception e) {
            throw new WsGenException("Cannot initialize the Velocity engine", e);
        }
	fileCptr = 0;
    }

    public void configureEndpoint(WSDeploymentDesc dd)
	throws WsGenException {
       
	// create deploy.wsdd
	// use velocity template
       	String tmplName = "deploy_endpoint.vm";
	String srcFileName = genDir.getPath()+File.separator+"server-config.wsdd";
	VelocityContext vctx = VContextFactory.createEndpointCtx(dd);
      	genFile(srcFileName, tmplName, vctx);
	
    }

    protected void configureClient(ServiceRef servRef)
	throws WsGenException {
	
	// the source generation is possible only when a
	// WSDL Definition is provided
	if (servRef.getWsdlFileName() != null 
	    && !servRef.getServiceInterface().getName().equals("javax.xml.rpc.Service") ) {
	    // Source generation for the client
	    String cmdJava = javaHomeBin + "java";
	    String wsdl = null;
	    
	    // Command preparation
	    Cmd axisCmd = new Cmd(cmdJava);
	    axisCmd.addArgument("-classpath");
	    axisCmd.addArgument(cp);
	    axisCmd.addArgument("org.apache.axis.wsdl.WSDL2Java");
	    
	    // get informations from ServiceRef ...
	    
	    // WSDL file
	    try {
		String pathWSDL = new File(filename).getCanonicalPath();
		wsdl = "jar:file://" + pathWSDL + "!/" + servRef.getWsdlFileName();
	    }
	    catch (IOException ioe) {
		String err = "Cannot retrieve file '" + filename + "'";
		throw new WsGenException(err, ioe);
	    }
	    
	    // Mapping File
	    MappingFile mf = servRef.getMappingFile();

	    for (Iterator i = mf.getMappings().keySet().iterator() ; i.hasNext() ; ) {
		String namespace = (String) i.next();
		String pkg = (String) mf.getMappings().get(namespace);
		axisCmd.addArgument("--NStoPkg");
		axisCmd.addArgument(namespace + "=" + pkg);
	    }

	    // Set the output directory
	    axisCmd.addArgument("--output");
	    axisCmd.addArgument(genDir.getPath());

	    if (isVerbose())
		axisCmd.addArgument("--verbose");

	    if (isDebug())
		axisCmd.addArgument("--Debug");

	
	    axisCmd.addArgument(wsdl);

//	    trace("Running '" + axisCmd.toString() + "'");
	    if (axisCmd.run()) {
		WsGen.info("Web Services Classes successfully generated by Axis.");
	    }
	    else {
		String err = "Error when running axis WSDL2Java tool.";
		throw new WsGenException(err);
	    }	    

	}// End source generation

	
	// Client-config.wsdd ??
	// only if handlers are specified in ServiceRef
	// but if noConfig set to true, assume that
	// user configure himself the client. (expert)
	
	// Could create configuration
	if (!isNoConfig() && (servRef.getHandlerRefs().size() != 0)) {
	    // create deploy.wsdd
	    // use velocity template
	    String tmplName = "deploy_client.vm";
	    // build a unique file name
	    fileCptr ++;
	    String srcFileName = genDir.getPath()+File.separator+"client-deploy-"+fileCptr+".wsdd";
	    VelocityContext vctx = VContextFactory.createClientCtx(servRef);
	    genFile(srcFileName, tmplName, vctx);	    
	} // End !noConfig

    }

    /**
     * return the webservices engine class name.
     * @return the webservices engine class name.
     */
    public String getEngineWebApp(){
	return "org.apache.axis.transport.http.AxisServlet";
    }


    private void genFile(String srcFileName, String tmplName, VelocityContext vctx)
	throws WsGenException {

	Template tmpl = null;
        FileWriter fwriter = null;		
	trace("create :"+srcFileName);
	try {
	    tmpl = vEngine.getTemplate(tmplName);
        } catch (Exception e) {
            e.printStackTrace();	    
            throw new WsGenException("Cannot get the '" + tmplName + "' template file", e);
        }
	System.out.println("end create !! ");
	
	try {
            // Create before the parent directory
            File fs = new File(srcFileName);
            File fdir = fs.getParentFile();
            if (fdir != null) {
                if (!fdir.exists()) {
                    if (!fdir.mkdirs()) {
                        throw new IOException("Cannot create the directory '" + fdir.getPath() + "'");
                    }
                }
            }
            fwriter = new FileWriter(srcFileName);
        } catch (IOException e) {
            e.printStackTrace();
            throw new WsGenException("Cannot create the '" + srcFileName
                                     + "' file", e);
        }

	trace("Generate the file " + srcFileName);
        try {
            tmpl.merge(vctx, fwriter);
        } catch (Exception e) {
            throw new WsGenException("Cannot generate the '" + srcFileName
                                     + "' file from the '" + tmplName + "' template", e);
        }

        try {
            fwriter.flush();
            fwriter.close();
        } catch (IOException e) {
            throw new WsGenException("Cannot close the '" + srcFileName
                                     + "' file", e);
        }

    }
}
