/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: ApplicationXml.java,v 1.1 2003/09/19 15:07:48 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen.xml;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.LineSeparator;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import org.objectweb.jonas_lib.deployment.SAXParserHelper;

public class ApplicationXml extends Xml {

    private static String APP_TAG = "application";
    private static String MODULE_TAG = "module";

    private static String WEB_TAG = "web";
    private static String URI_TAG = "web-uri";
    private static String CTX_TAG = "context-root";

    private static String EJB_TAG = "ejb";
    
    public ApplicationXml (InputStream is) throws XmlException {
	super(is);
    }

    public void removeModules()
    throws XmlException {
	try {
	Document core = getDocument();
	Element root = core.getDocumentElement();
	
	NodeList nl = root.getElementsByTagName(MODULE_TAG);
	for(int i = 0; i < nl.getLength(); i++) {
	   root.removeChild(nl.item(i));
	}
	}
	catch (Exception e) {
	    throw new XmlException("Error when remove module nodes",e);
	}
    }

    public void addWeb(String webName, String ctx){

	Document core = getDocument();
	Element root = core.getDocumentElement();

	// build nodes
	Node uriTxt = core.createTextNode(webName);
	Node ctxTxt = core.createTextNode(ctx);
	Element uriE = core.createElement(URI_TAG);
	Element ctxE = core.createElement(CTX_TAG);
	Element webE = core.createElement(WEB_TAG);	
	Element newNode = core.createElement(MODULE_TAG);

	uriE.appendChild(uriTxt);
	ctxE.appendChild(ctxTxt);
	webE.appendChild(uriE);
	webE.appendChild(ctxE);
	newNode.appendChild(webE);
	root.appendChild(newNode);	
    }

    public void addEjb(String ejbName){

	Document core = getDocument();
	Element root = core.getDocumentElement();

	// build nodes
	Node ejbNameTxt = core.createTextNode(ejbName);
	Element ejbNameE = core.createElement(EJB_TAG);
	Element newNode = core.createElement(MODULE_TAG);

	ejbNameE.appendChild(ejbNameTxt);
	newNode.appendChild(ejbNameE);
	root.appendChild(newNode);	
    }

}
