/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: F_ClientViewSF.java,v 1.6 2003/07/02 06:07:36 legrasi Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtests.clients.session;

import junit.framework.Test;
import junit.framework.TestSuite;
import javax.rmi.PortableRemoteObject;
import org.objectweb.jonas.jtests.beans.local.Target;
import org.objectweb.jonas.jtests.beans.local.TargetSFHome;
import org.objectweb.jonas.jtests.beans.local.TargetSLHome;
import org.objectweb.jonas.jtests.util.JTestCase;


/**
 * testcases  specifics for stateful session beans must be here
 * 
 * - isIdentical, create<Method>
 */
public class F_ClientViewSF extends A_ClientView {  

    protected static String BEAN_HOME = "EJB/localTargetSFHome";
    protected static TargetSFHome home = null;

    public F_ClientViewSF(String name) {
        super(name);
    }
  
    /**
     * test EJBObject.isIdentical on two beans
     * @see Spec EJB2.0 6.9.1
     */
    public void testIsIdenticalOnBeans() throws Exception {	
        Target tr1 = (Target) getHome().create();
        Target tr2 = (Target) getHome().create();
        assertTrue(!tr1.isIdentical(tr2));
        tr1.remove();
        tr2.remove();
    }

    /**
     * test create with parameters
     */
    public void testCreateWithParameter() throws Exception {
        String s1 = "String1";
        TargetSFHome tsfh = (TargetSFHome)getHome();
        Target tr = tsfh.create(s1, 1000);
        assertEquals(tr.getString(), s1);
        assertEquals(tr.getNumber(), 1000);
        assertTrue(!tr.isCreatedViaCreateXX());
        tr.remove();
    }

    /**
     * test create<METHOD> 
     * 
     */
    public void testCreateMethod() throws Exception {
        String s2 = "String2";
        TargetSFHome tsfh = (TargetSFHome)getHome();
        Target tr = tsfh.createXX(s2,2000);
        assertEquals(tr.getString(),s2);
        assertEquals(tr.getNumber(),2000);
        assertTrue(tr.isCreatedViaCreateXX());
        tr.remove();
    }


    /**
     * init environment:
     */
    protected void setUp() {
        super.setUp();
        useBeans("local", false);
    }

    public TargetSLHome getHome() throws Exception {
        if (home == null) {
            home = (TargetSFHome) PortableRemoteObject.narrow(ictx.lookup(BEAN_HOME),TargetSFHome.class) ;
        }
        assertTrue(home != null);
        return home;
    }

    public static Test suite() {
        return new TestSuite(F_ClientViewSF.class);
    }

    protected static void callTest(String testname) throws Exception {
        System.out.println("---------------------------------------------");
        JTestCase.callTest("session.F_ClientViewSF", testname);
    }
    
    protected static void callrepeatedTest( String testname) throws Exception {
        System.out.println("---------------------------------------------");
        JTestCase.callTest("session.F_ClientViewSF", testname);
        for (int i = 1; i < 120; i = i * 10) {
            JTestCase.callrepeatedTest("session.F_ClientViewSF", testname, i);
        }
    }
    /**
     * Run tests for perf mesures
     */
    protected static void perfs() throws Exception {
      
        System.out.println ("------> Bean TargetSF <--------");

        callTest("testEmpty");

        callrepeatedTest("testCreateRemove");
    
        callrepeatedTest("testBusinessMethod2");
  
        callrepeatedTest("testBusinessMethod3");

        callrepeatedTest("testCreateMethod");

        callrepeatedTest("testCreateWithParameter");
    }

    public static void main (String args[]) throws Exception  {
        String testtorun = null;
        // Get args
        for (int argn = 0; argn < args.length; argn++) {
            String s_arg = args[argn];
            Integer i_arg;
            if (s_arg.equals("-n")) {
                testtorun = args[++argn];
            }
        }
        if (testtorun == null) {
            junit.textui.TestRunner.run(suite());
        } else { if (testtorun.equals("perfs")) {
		perfs();
	    } else {  
            junit.textui.TestRunner.run(new F_ClientViewSF(testtorun));
        }            
    }
    }
}
